# Generated by Django 5.0.13 on 2025-03-20 04:42

from django.db import migrations, models
from django.apps import apps

from openbook_common.utils.model_loaders import get_appointment_model


def appointments_table_name() -> str:
    AppointmentsModel = apps.get_model("openbook_appointments.Appointment")
    return AppointmentsModel._meta.db_table


def location_details_table_name() -> str:
    LocationDetailsModel = apps.get_model("openbook_appointments.LocationDetails")
    return LocationDetailsModel._meta.db_table


def migration_sql(table_name) -> str:
    return f"""
        UPDATE {table_name}
        SET geolocation_geojson = jsonb_build_object(
            'type', 'Point',
            'coordinates', ARRAY[
                ST_X(geolocation::geometry),
                ST_Y(geolocation::geometry)
            ]
        )
        WHERE geolocation IS NOT NULL;
    """


def reverse_migration_sql(table_name) -> str:
    return f"""
        UPDATE {table_name}
        SET geolocation_geojson = NULL;
    """


def run_migration(apps, schema_editor):
    schema_editor.execute(migration_sql(appointments_table_name()))
    schema_editor.execute(migration_sql(location_details_table_name()))


def reverse_migration(apps, schema_editor):
    schema_editor.execute(reverse_migration_sql(location_details_table_name()))
    schema_editor.execute(reverse_migration_sql(appointments_table_name()))


class Migration(migrations.Migration):
    dependencies = [
        ("openbook_appointments", "0009_appointment_deleted_at_appointment_is_deleted"),
    ]

    operations = [
        migrations.AddField(
            model_name="appointment",
            name="geolocation_geojson",
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name="locationdetails",
            name="geolocation_geojson",
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.RunPython(run_migration, reverse_migration),
    ]
